# Generated by Django 3.0.9 on 2020-11-03 10:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("photograph", "0005_implement_deep_zoom_fields"),
    ]

    operations = [
        migrations.AddField(
            model_name="photograph",
            name="profile_position",
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
    ]
