# Generated by Django 3.0.7 on 2020-07-21 14:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("message", "0002_correct_typo"),
    ]

    operations = [
        migrations.AlterField(
            model_name="message",
            name="valid_to",
            field=models.DateField(blank=True, null=True),
        ),
    ]
