# Generated by Django 3.0.9 on 2020-10-29 15:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("slideshow", "0008_add_active_and_position_field_slideshow_model"),
    ]

    operations = [
        migrations.AlterField(
            model_name="slideshow",
            name="position",
            field=models.PositiveSmallIntegerField(),
        ),
        migrations.AlterField(
            model_name="slideshowimage",
            name="position",
            field=models.PositiveSmallIntegerField(),
        ),
        migrations.AlterField(
            model_name="slideshowpage",
            name="position",
            field=models.PositiveSmallIntegerField(),
        ),
    ]
