# Generated by Django 3.0.9 on 2020-08-24 11:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("photograph", "0004_add_verbose_names"),
    ]

    operations = [
        migrations.AddField(
            model_name="photograph",
            name="dzi_file",
            field=models.FileField(
                editable=False, null=True, upload_to="", verbose_name="Deep Zoom file"
            ),
        ),
        migrations.AddField(
            model_name="photograph",
            name="dzi_option",
            field=models.BooleanField(default=False, verbose_name="Deep Zoom option"),
        ),
    ]
