# Generated by Django 3.0.7 on 2020-07-28 14:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("photograph", "0003_create_generic_foreign_key"),
    ]

    operations = [
        migrations.AlterField(
            model_name="photograph",
            name="img_original_height",
            field=models.PositiveSmallIntegerField(
                editable=False, verbose_name="img height"
            ),
        ),
        migrations.AlterField(
            model_name="photograph",
            name="img_original_width",
            field=models.PositiveSmallIntegerField(
                editable=False, verbose_name="img width"
            ),
        ),
    ]
